@@ -30,11 +30,12 @@ using System.Collections;
 
 using Gnome;
 using Gtk;
+using GD=Gdk;
 
-using Mono.Posix;
+using Mono.Unix;
 
 using Beagle;
-using BU = Beagle.Util;
+using Beagle.Util;
 using Beagle.Tile;
 
 namespace Best {
@@ -62,8 +63,8 @@ namespace Best {
 		}
 		
 		public void FocusEntry () {
-			entry.SelectRegion (0, -1);
-			entry.GtkEntry.GrabFocus ();
+			//entry.SelectRegion (0, -1);
+			entry.GrabFocus ();
 		}
 
 		public void FocusEntryHandler (object o, EventArgs args) {
@@ -71,7 +72,7 @@ namespace Best {
 		}
 
 		Gtk.AccelGroup accel_group;
-		BU.GlobalKeybinder global_keys;
+		GlobalKeybinder global_keys;
 
 		void CreateWindow (string query)
 		{
@@ -101,7 +102,7 @@ namespace Best {
 
 			accel_group = new Gtk.AccelGroup ();
 			this.AddAccelGroup (accel_group);
-			global_keys = new BU.GlobalKeybinder (accel_group);
+			global_keys = new GlobalKeybinder (accel_group);
 
 			// Close window (Ctrl-W)
 			global_keys.AddAccelerator (new EventHandler (this.HideWindowHandler),
@@ -139,8 +140,10 @@ namespace Best {
 						    0,
 						    Gtk.AccelFlags.Visible);
 
-			DBusisms.BeagleDown += OnBeagleDown;
+			//DBusisms.BeagleDown += OnBeagleDown;
 
+			UpdateFromConf ();
+			
 			UpdatePage ();
 
 			if (query != null)
@@ -152,12 +155,11 @@ namespace Best {
 
 		int posX = 0, posY = 0;
 
-		public new void Show ()
+		public new void Present ()
 		{
-			base.Show ();	
+			base.Present ();	
 			
 			Move (posX, posY);
-			Present ();
 		}
 
 		public new void Hide ()
@@ -227,6 +229,8 @@ namespace Best {
 			sw.HscrollbarPolicy = Gtk.PolicyType.Automatic;
 			sw.VscrollbarPolicy = Gtk.PolicyType.Automatic;
 			Gtk.TextView view = new Gtk.TextView ();
+			view.CursorVisible = false;
+			view.Editable = false;
 			Gtk.TextBuffer buffer = view.Buffer;
 			buffer.Text = canvas.Source;
 			view.Buffer = buffer;
@@ -267,9 +271,11 @@ namespace Best {
 
 		//////////////////////////
 
-		private Gnome.Entry entry;
+		private Gtk.Entry entry;
+		private Gtk.ListStore history;
+
+		private Gtk.ListStore filter_data;
 		
-		private Gtk.ScrolledWindow swin;
 		private TileCanvas canvas;
 		private SimpleRootTile root;
 		private Gtk.Label page_label;
@@ -293,51 +299,70 @@ namespace Best {
 			return button;
 		}
 
-		private Gtk.OptionMenu TypesOptionMenu ()
+		private Gtk.ComboBox FilterComboBox ()
 		{
-			Gtk.OptionMenu opt = new Gtk.OptionMenu ();
+			filter_data = new Gtk.ListStore (new Type[] { typeof (string), typeof (string) });
+			
+			Gtk.TreeIter iter;
 
-			ArrayList items = new ArrayList ();
-			Gtk.MenuItem mi;
-			mi = new TypeMenuItem (Catalog.GetString ("Anywhere"), null);
-			items.Add (mi);
-			mi = new TypeMenuItem (Catalog.GetString ("in Files"), "File");
-			items.Add (mi);
-			mi = new TypeMenuItem (Catalog.GetString ("in Addressbook"), "Contact");
-			items.Add (mi);
-			mi = new TypeMenuItem (Catalog.GetString ("in Mail"), "MailMessage");
-			items.Add (mi);
-			mi = new TypeMenuItem (Catalog.GetString ("in Web Pages"), "WebHistory");
-			items.Add (mi);
-			mi = new TypeMenuItem (Catalog.GetString ("in Chats"), "IMLog");
-			items.Add (mi);
+			iter = filter_data.Append ();
+			filter_data.SetValue (iter, 0, Catalog.GetString ("Anywhere"));
+			filter_data.SetValue (iter, 1, null);
 
-			Gtk.Menu menu = new Gtk.Menu ();
-			foreach (Gtk.MenuItem item in items)
-				menu.Append (item);
-			opt.Menu = menu;
-			opt.Data["items"] = items;
-		
-			return opt;
+			iter = filter_data.Append ();
+			filter_data.SetValue (iter, 0, Catalog.GetString ("in Files"));
+			filter_data.SetValue (iter, 1, "File");
+
+			iter = filter_data.Append ();
+			filter_data.SetValue (iter, 0, Catalog.GetString ("in Addressbook"));
+			filter_data.SetValue (iter, 1, "Contact");
+
+			iter = filter_data.Append ();
+			filter_data.SetValue (iter, 0, Catalog.GetString ("in Mail"));
+			filter_data.SetValue (iter, 1, "MailMessage");
+
+			iter = filter_data.Append ();
+			filter_data.SetValue (iter, 0, Catalog.GetString ("in Web Pages"));
+			filter_data.SetValue (iter, 1, "WebHistory");
+
+			iter = filter_data.Append ();
+			filter_data.SetValue (iter, 0, Catalog.GetString ("in Chats"));
+			filter_data.SetValue (iter, 1, "IMLog");
+
+			Gtk.ComboBox combo = new Gtk.ComboBox (filter_data);
+			combo.Active = 0;
+
+			Gtk.CellRendererText renderer = new Gtk.CellRendererText ();
+			combo.PackStart (renderer, false);
+			combo.SetAttributes (renderer, new object[] { "text", 0 });
+
+			return combo;
 		}
-
+		
 		private Gtk.Widget CreateContents ()
 		{
-			Gtk.HBox entryLine = new HBox (false, 3);
+			Gtk.HBox entryLine = new HBox (false, 4);
 
 			Gtk.Label words = new Gtk.Label (Catalog.GetString ("Search terms:"));
 			entryLine.PackStart (words, false, false, 3);
+
+			history = new Gtk.ListStore (new Type[] { typeof (string) });
+
+			Gtk.EntryCompletion comp = new Gtk.EntryCompletion ();
+			comp.Model = history;
+			comp.TextColumn = 0;
 			
-			entry = new Gnome.Entry ("");
+			entry = new Gtk.Entry ("");
 			entry.Activated += new EventHandler (this.DoSearch);
+			entry.Completion = comp;
 			entryLine.PackStart (entry, true, true, 3);
 
 			words = new Gtk.Label ("");
 			entryLine.PackStart (words, false, false, 3);
 
-			Gtk.OptionMenu types = TypesOptionMenu ();
-			types.Changed += new EventHandler (this.ChangeType);
-			entryLine.PackStart (types, false, false, 3);
+			Gtk.ComboBox combo = FilterComboBox ();
+			combo.Changed += new EventHandler (this.ChangeType);
+			entryLine.PackStart (combo, false, false, 3);
 
 			Gtk.HBox buttonContents = new HBox (false, 0);
 			Gtk.Widget buttonImg = Beagle.Images.GetWidget ("icon-search.png");
@@ -349,6 +374,11 @@ namespace Best {
 			button.Add (buttonContents);
 			button.Clicked += new EventHandler (this.DoSearch);
 			entryLine.PackStart (button, false, false, 3);
+			
+			Gtk.Button clearButton = new Gtk.Button ();
+			clearButton.Label = "Clear";
+			clearButton.Clicked += new EventHandler (this.ClearSearch);
+			entryLine.PackStart (clearButton, false, false, 4);
 
 			canvas = new TileCanvas ();
 			canvas.Show ();
@@ -384,6 +414,56 @@ namespace Best {
 			return contents;
 		}
 
+		private void UpdateFromConf ()
+		{
+			Console.WriteLine ("Reading settings from Config");
+			// FIXME: there might be weird cases with multiple screens,
+			// multiple monitors, resolution related problems that might
+			// cause problem is remapping the stored values to current
+			// screen coordinates
+			int res_x = GD.Screen.Default.Width;
+			int res_y = GD.Screen.Default.Height;
+			int pos_x = (int)(Conf.Searching.BestPosX * res_x / 100);
+			int pos_y = (int)(Conf.Searching.BestPosY * res_y / 100);
+			int width = (int)(Conf.Searching.BestWidth * res_x / 100 );
+			int height = (int)(Conf.Searching.BestHeight * res_y / 100);
+			
+			if (pos_x != 0 || pos_y != 0) {
+				posX = pos_x;
+				posY = pos_y;
+				Move (pos_x, pos_y);
+			}
+			if (width != 0)
+				DefaultWidth = width;
+			if (height != 0)
+				DefaultHeight = height;
+			Gtk.TreeIter iter;
+			foreach (string search in Conf.Searching.SearchHistory) {
+				iter = history.Append ();
+				history.SetValue (iter, 0, search);
+			}
+		}
+
+		public void StoreSettingsInConf ()
+		{
+			Console.WriteLine ("Storing setting in Config");
+			int pos_x = posX, pos_y = posY;
+			if (WindowIsVisible)
+				GetPosition (out pos_x, out pos_y);
+			int width = 0, height = 0;
+			GetSize (out width, out height);
+			
+			int res_x = GD.Screen.Default.Width;
+			int res_y = GD.Screen.Default.Height;
+			Conf.Searching.BestPosX = ((float) pos_x / res_x) * 100;
+			Conf.Searching.BestPosY = ((float) pos_y / res_y) * 100;
+			Conf.Searching.BestWidth = ((float) width / res_x) * 100;
+			Conf.Searching.BestHeight = ((float) height / res_y) * 100;
+			Conf.Searching.SearchHistory = RetriveSearches ();
+			Conf.Searching.SaveNeeded = true;
+			Conf.Save ();
+		}
+
 		private void UpdatePage ()
 		{
 			//Console.WriteLine ("In UpdatePage");
@@ -424,72 +504,65 @@ namespace Best {
 				string tmp = delayedQuery;
 				delayedQuery = null;
 				System.Console.WriteLine ("Delayed query fired");
-				Search (tmp);
+				StartQuery ();
 			}
 
 			return false;
 		}
 
-		private void QueueDelayedQuery ()
+		private void QueueDelayedQuery (string query)
 		{
-			GLib.Timeout.Add (1000, new GLib.TimeoutHandler (RunDelayedQuery));
+			delayedQuery = query;
+			GLib.Timeout.Add (10000, new GLib.TimeoutHandler (RunDelayedQuery));
 		}
 		
 		private void DoSearch (object o, EventArgs args)
 		{
-			if (entry.GtkEntry.Text != null && entry.GtkEntry.Text != "")
-				Search (entry.GtkEntry.Text);
+			if (entry.Text != null && entry.Text != "")
+				Search (entry.Text);
 		}
 
-		private string lastType = null;
+		//private string lastType = null;
 
 		private void ChangeType (object o, EventArgs args)
 		{
-			Gtk.OptionMenu opt = (Gtk.OptionMenu) o;
+			Gtk.ComboBox combo = (Gtk.ComboBox) o;
+			string hit_type = null;
+			Gtk.TreeIter iter;
 
-			TypeMenuItem mi = (TypeMenuItem) ((ArrayList) opt.Data["items"])[opt.History];
+			if (combo.GetActiveIter (out iter))
+				hit_type = (string) filter_data.GetValue (iter, 1);
 
-			this.hit_type = mi.Type;
+			if (this.hit_type == hit_type)
+				return;
+
+			this.hit_type = hit_type;
 
 			root.SetSource (this.hit_type);
-
 			root.HitCollection.PageFirst ();
 			UpdatePage ();
-
-			//if (this.query != null && lastType != this.hit_type) {
-			//	Search (entry.GtkEntry.Text);
-			//	lastType = this.hit_type;
-			//}
-			//UpdatePage ();
 		}
 
 		//////////////////////////
 
-		private void Close ()
+		private void OnHitsAdded (HitsAddedResponse response)
 		{
-			Destroy ();
-		}
-
-		//////////////////////////
-
-		private void OnHitsAdded (Query source, ICollection hits)
-		{
-			root.Add (hits);
+			root.Add (response.Hits);
 			UpdatePage ();
 		}
 
-		private void OnHitsSubtracted (Query source, ICollection uris)
+		private void OnHitsSubtracted (HitsSubtractedResponse response)
 		{
-			root.Subtract (uris);
+			root.Subtract (response.Uris);
 			UpdatePage ();
 		}
 
-		private void OnFinished (QueryProxy source)
+		private void OnFinished (FinishedResponse repsonse)
 		{
 			SetBusy (false);
 		}
 
-		private void OnCancelled (QueryProxy source)
+		private void OnCancelled (CancelledResponse response)
 		{
 			SetBusy (false);
 		}
@@ -509,84 +582,104 @@ namespace Best {
 				query.HitsSubtractedEvent -= OnHitsSubtracted;
 				query.FinishedEvent -= OnFinished;
 				query.CancelledEvent -= OnCancelled;
-				query.Dispose ();
+				query.Close ();
+
 				query = null;
 			}
 		}
 
+		private void StartQuery ()
+		{
+			try {
+				query.SendAsync ();
+				SetBusy (true);
+			} catch (Beagle.ResponseMessageException e) {
+				QueueDelayedQuery (entry.Text);
+				
+				/* To translators: {0} represents the current query keywords */
+				root.Error (String.Format (Catalog.GetString ("The query for <i>{0}</i> failed." +
+						    "<br>The likely cause is that the beagle daemon isn't running."), entry.Text));
+				root.OfferDaemonRestart = true;
+			} catch (Exception e) {
+				/* To translators: {0} represents the current query keywords, {1} contains the errormessage */
+				root.Error (String.Format (Catalog.GetString ("The query for <i>{0}</i> failed with error:<br>{1}<br>"), 
+							   entry.Text, e));
+			}
+		}
+
 		private void Search (String searchString)
 		{
-			try {
-				StoreSearch (searchString);
-				entry.GtkEntry.Text = searchString;
+			entry.Text = searchString;
+			StoreSearch (searchString);
 
-				if (query != null) {
-					query.Cancel ();
-					DetachQuery ();
-				}
-
-				query = Factory.NewQuery ();
-				query.AddDomain (QueryDomain.Neighborhood);
-
-				// FIXME: Disable non-local searching for now.
-				//query.AddDomain (QueryDomain.Global);
-
-				query.AddText (searchString);
-				root.SetSource (hit_type);
-				
-				AttachQuery ();
-				
-				root.Query = query;
-				root.Start ();
-				
-				SetBusy (true);
-				query.Start ();
-			} catch (Exception e) {
-				delayedQuery = entry.GtkEntry.Text;
-				DBusisms.BeagleUpAgain += QueueDelayedQuery;
-
-				if (e.ToString ().IndexOf ("com.novell.Beagle") != -1) {
-					/* To translators: {0} represents the current query keywords */
-					root.Error (String.Format (Catalog.GetString ("The query for <i>{0}</i> failed." +
-						    "<br>The likely cause is that the beagle daemon isn't running."),searchString));
-					root.OfferDaemonRestart = true;
-				} else if (e.ToString().IndexOf ("Unable to determine the address") != -1) {
-					/* To translators: {0} represents the current query keywords */
-					root.Error (String.Format (Catalog.GetString ("The query for <i>{0}</i> failed.<br>" +
-						    "The session bus isn't running.  See http://beaglewiki.org/index.php/Installing%20Beagle for information on setting up a session bus."),searchString));
-				} else {
-					/* To translators: {0} represents the current query keywords, {1} contains the errormessage */
-					root.Error (String.Format (Catalog.GetString ("The query for <i>{0}</i> failed with error:<br>{1}<br>"),searchString, e));
-				}
+			if (query != null) {
+				try { DetachQuery (); } catch (ObjectDisposedException e) {}
 			}
-
+			
+			query = new Query ();
+			query.AddDomain (QueryDomain.Neighborhood);
+			
+			// FIXME: Disable non-local searching for now.
+			//query.AddDomain (QueryDomain.Global);
+			
+			query.AddText (searchString);
+			root.SetSource (hit_type);
+			
+			AttachQuery ();
+			
+			root.Query = query;
+			root.Start ();
+			
+			StartQuery ();
+			
 			UpdatePage ();
 		}
-
-		ArrayList recentSearches;
 		
-		private void StoreSearch (string newQuery)
+		private void ClearSearch (object o, EventArgs args)
 		{
-			if (recentSearches == null) {
-				recentSearches = new ArrayList ();
+			root.Clear ();
+			UpdatePage ();
+			entry.Text = "";
+		}
+
+		private void StoreSearch (string query)
+		{
+			Gtk.TreeIter iter;
+
+			if (history.GetIterFirst (out iter)) {
+				string val;
+
+				do {
+					val = (string) history.GetValue (iter, 0);
+
+					if (val == query)
+						history.Remove (ref iter);
+				} while (val != query && history.IterNext (ref iter));
 			}
 
-			if (newQuery != null && newQuery != "") {
-				int duplicate = recentSearches.IndexOf (newQuery);
-				if (duplicate >= 0) {
-					recentSearches.RemoveAt (duplicate);
-				}
-				recentSearches.Insert (0, newQuery);				
-			}
-
-			if (recentSearches.Count == 11) {
-				recentSearches.RemoveAt (10);
-			}
+			iter = history.Insert (0);
+			history.SetValue (iter, 0, query);
 		}
 		
 		public ArrayList RetriveSearches () 
 		{
-			return recentSearches;
+			ArrayList searches = new ArrayList ();
+
+			int i = 0;
+			foreach (object[] o in history) {
+				searches.Add (o[0]);
+				i++;
+
+				if (i == 10)
+					break;
+			}
+
+			return searches;
+		}
+
+		public void ClearHistory ()
+		{
+			history.Clear ();
 		}
 		
 		public void QuickSearch (string query) 
