@@ -29,8 +29,6 @@ using System;
 using System.Collections;
 using System.IO;
 
-using Mono.Posix;
-
 using GMime;
 
 using Beagle;
@@ -64,7 +62,7 @@ namespace Beagle.Filters {
 				}
 			}
 
-			int mail_fd = Syscall.open (info.FullName, OpenFlags.O_RDONLY);
+			int mail_fd = Mono.Unix.Syscall.open (info.FullName, Mono.Unix.OpenFlags.O_RDONLY);
 			
 			if (mail_fd == -1)
 				throw new IOException (String.Format ("Unable to read {0} for parsing mail", info.FullName));
@@ -80,14 +78,14 @@ namespace Beagle.Filters {
 			string subject = GMime.Utils.HeaderDecodePhrase (this.message.Subject);
 			AddProperty (Property.New ("dc:title", subject));
 
-			AddProperty (Property.NewDate ("fixme:date", message.Date));
+			AddProperty (Property.NewDate ("fixme:date", message.Date.ToUniversalTime ()));
 
 			GMime.InternetAddressList addrs;
 			addrs = this.message.GetRecipients (GMime.Message.RecipientType.To);
 			foreach (GMime.InternetAddress ia in addrs) {
 				AddProperty (Property.NewKeyword ("fixme:to", ia.ToString (false)));
 				AddProperty (Property.NewKeyword ("fixme:to_address", ia.Addr));
-				AddProperty (Property.NewKeyword ("fixme:to_name", ia.Name));
+				AddProperty (Property.New ("fixme:to_name", ia.Name));
 			}
 			addrs.Dispose ();
 
@@ -95,7 +93,7 @@ namespace Beagle.Filters {
 			foreach (GMime.InternetAddress ia in addrs) {
 				AddProperty (Property.NewKeyword ("fixme:cc", ia.ToString (false)));
 				AddProperty (Property.NewKeyword ("fixme:cc_address", ia.Addr));
-				AddProperty (Property.NewKeyword ("fixme:cc_name", ia.Name));
+				AddProperty (Property.New ("fixme:cc_name", ia.Name));
 			}
 			addrs.Dispose ();
 
@@ -103,7 +101,7 @@ namespace Beagle.Filters {
 			foreach (GMime.InternetAddress ia in addrs) {
 				AddProperty (Property.NewKeyword ("fixme:from", ia.ToString (false)));
 				AddProperty (Property.NewKeyword ("fixme:from_address", ia.Addr));
-				AddProperty (Property.NewKeyword ("fixme:from_name", ia.Name));
+				AddProperty (Property.New ("fixme:from_name", ia.Name));
 			}
 			addrs.Dispose ();
 
@@ -222,7 +220,7 @@ namespace Beagle.Filters {
 						string sub_uri = this.filter.Uri.ToString () + "#" + this.count;
 						Indexable child = new Indexable (new Uri (sub_uri));
 
-						child.Type = "MailMessage";
+						child.HitType = "MailMessage";
 						child.MimeType = part.ContentType.ToString ();
 						child.CacheContent = false;
 
