@@ -28,15 +28,19 @@ using System;
 using System.Diagnostics;
 using System.IO;
 
-using BU = Beagle.Util;
+using Beagle.Util;
+
+using Gnome;
+using Gdk;
+using Gtk;
 
 namespace Beagle.Tile {
 
 	[HitFlavor (Name="Files", Rank=300, Emblem="emblem-file.png", Color="#f5f5fe",
-		    Type="File"),
-	 HitFlavor (Name="Files", Rank=300, Emblem="emblem-file.png", Color="#f5f5fe",
-		    Uri="file://*")]
+		    Type="File")]
 	public class TileFile : TileFromHitTemplate {
+		private static ThumbnailFactory thumb_factory = new ThumbnailFactory (ThumbnailSize.Normal);
+
 		public TileFile (Hit _hit) : base (_hit,
 						   "template-file.html")
 		{
@@ -56,13 +60,34 @@ namespace Beagle.Tile {
 				return;
 			}
 
-			Gtk.IconSize size = (Gtk.IconSize) 48;
-			string path = BU.GnomeIconLookup.LookupMimeIcon (Hit.MimeType,
-									 size);
-			if (path != null) {
-				Template["Icon"] = Images.GetHtmlSource (path,
-									 BU.GnomeIconLookup.GetFileMimeType (path));
+			string quoted_uri = StringFu.PathToQuotedFileUri (Hit.Uri.LocalPath);
+			string thumbnail = Thumbnail.PathForUri (quoted_uri, ThumbnailSize.Normal);
+
+			if (File.Exists (thumbnail))
+				Template ["Icon"] = Images.GetHtmlSource (thumbnail, Hit.MimeType);
+			else {
+				Pixbuf pix = thumb_factory.GenerateThumbnail (quoted_uri, Hit.MimeType);
+				FileInfo fi = new FileInfo (Hit.Uri.LocalPath);
+				if (pix == null) {
+					thumb_factory.CreateFailedThumbnail (quoted_uri, fi.LastWriteTime);
+					
+					string path = GnomeFu.GetMimeIconPath (Hit.MimeType);
+					string icon = Images.GetHtmlSource (path, GnomeFu.GetMimeType (path));
+
+					if (icon != null)
+						Template ["Icon"] = icon;
+					else
+						Template ["Icon"] = Images.GetHtmlSource ("document", "image/png");
+				} else {
+					thumb_factory.SaveThumbnail (pix, quoted_uri, DateTime.Now);
+					Template ["Icon"] = Images.GetHtmlSource (thumbnail, Hit.MimeType);
+				}
 			}
+
+			Template["Title"] = Hit ["dc:title"];
+
+			if (Template["Title"] == null)
+				Template["Title"] = Template["FileName"];
 		}
 
 		[TileAction]
@@ -74,13 +99,13 @@ namespace Beagle.Tile {
 		[TileAction]
 		public void SendTo ()
 		{
-			SendFile (String.Format ("'{0}'", Hit.Uri.LocalPath));
+			SendFile (String.Format ("{0}", Hit.Uri.LocalPath));
 		}
 
 		[TileAction]
 		public void Reveal ()
 		{
-			OpenFolder (Hit ["fixme:directory"]);
+			OpenFolder (Path.GetDirectoryName (Hit.Uri.LocalPath));
 		}
 	}
 }
