@@ -31,7 +31,7 @@ using System.IO;
 using System.Text;
 using System.Xml;
 
-using Mono.Posix;
+using Mono.Unix;
 
 namespace Beagle.Util {
 
@@ -46,6 +46,16 @@ namespace Beagle.Util {
 			return dt.ToString (timeFormat);
 		}
 
+		static public string DateTimeToYearMonthString (DateTime dt)
+		{
+			return dt.ToString ("yyyyMM");
+		}
+
+		static public string DateTimeToDayString (DateTime dt)
+		{
+			return dt.ToString ("dd");
+		}
+
                 static public DateTime StringToDateTime (string str)
                 {
 			if (str == null || str == "")
@@ -66,10 +76,13 @@ namespace Beagle.Util {
 			else if (sinceToday.TotalDays < 1)
 				date = Catalog.GetString ("Yesterday");
 			else if (today.Year == dt.Year)
+				/* Translators: Example output: Aug 9 */
 				date = dt.ToString (Catalog.GetString ("MMM d"));
 			else
+				/* Translators: Example output: Aug 9, 2000 */
 				date = dt.ToString (Catalog.GetString ("MMM d, yyyy"));
-			
+		
+			/* Translators: Example output: 11:05 AM  (note h = 12-hour time) */
 			time = dt.ToString (Catalog.GetString ("h:mm tt"));
 
 			string fuzzy;
@@ -97,16 +110,18 @@ namespace Beagle.Util {
 				} else if (date.DayOfYear == now.DayOfYear - 1) {
 					/* To translators: {0} is the time of the day, eg. 13:45 */
 					return String.Format (Catalog.GetString ("Yesterday, {0}"), short_time);
-				} else if (date.DayOfYear > now.DayOfYear - 6) {
+				} else if (date.DayOfYear > now.DayOfYear - 6 && date.DayOfYear < now.DayOfYear) {
 					/* To translators: {0} is the number of days that have passed, {1} is the time of the day, eg. 13:45 */
 					return String.Format (Catalog.GetString ("{0} days ago, {1}"),
 							      now.DayOfYear - date.DayOfYear,
 							      short_time);
 				} else {
+					/* Translators: Example output: January 3, 3:45 PM */
 					return date.ToString (Catalog.GetString ("MMMM d, h:mm tt"));
 				}
 			}
 
+			/* Translators: Example output: March 23 2001, 10:04 AM */
 			return date.ToString (Catalog.GetString ("MMMM d yyyy, h:mm tt"));
 		}
 		
@@ -150,57 +165,53 @@ namespace Beagle.Util {
 			return String.Format (Catalog.GetString ("{0:0.0} MB"), len/(double)oneMb);
 		}
 
-		// FIXME: This is pretty inefficient
-		static public string[] FuzzySplit (string line)
+		// Here we:
+		// (1) Replace non-alphanumeric characters with spaces
+		// (2) Inject whitespace between lowercase-to-uppercase
+		//     transitions (so "FooBar" becomes "Foo Bar")
+		//     and transitions between letters and numbers
+		//     (so "cvs2svn" becomes "cvs 2 svn")
+		static public string FuzzyDivide (string line)
 		{
-			int i;
+			// Allocate a space slightly bigger than the
+			// original string.
+			StringBuilder builder;
+			builder = new StringBuilder (line.Length + 4);
 
-			// Replace non-alphanumeric characters with spaces
-			StringBuilder builder = new StringBuilder (line.Length);
-			for (i = 0; i < line.Length; ++i) {
-				if (Char.IsLetterOrDigit (line [i]))
-					builder.Append (line [i]);
-				else
-					builder.Append (" ");
-			}
-			line = builder.ToString ();
+			int prev_case = 0;
+			bool last_was_space = true; // don't start w/ a space
+			for (int i = 0; i < line.Length; ++i) {
+				char c = line [i];
+				int this_case = 0;
+				if (Char.IsLetterOrDigit (c)) {
+					if (Char.IsUpper (c))
+						this_case = +1;
+					else if (Char.IsLower (c))
+						this_case = -1;
+					if (this_case != prev_case
+					    && !(this_case == -1 && prev_case == +1)) {
+						if (! last_was_space) {
+							builder.Append (' ');
+							last_was_space = true;
+						}
+					}
+					
+					if (c != ' ' || !last_was_space) {
+						builder.Append (c);
+						last_was_space = (c == ' ');
+					}
 
-			// Inject whitespace on all case changes except
-			// from upper to lower.
-			i = 0;
-			int prevCase = 0;
-			while (i < line.Length) {
-				int thisCase;
-				if (Char.IsUpper (line [i]))
-					thisCase = +1;
-				else if (Char.IsLower (line [i]))
-					thisCase = -1;
-				else
-					thisCase = 0;
-
-				if (prevCase != thisCase
-				    && !(prevCase == +1 && thisCase == -1)) {
-					line = line.Substring (0, i) + " " + line.Substring (i);
-					++i;
+					prev_case = this_case;
+				} else {
+					if (! last_was_space) {
+						builder.Append (' ');
+						last_was_space = true;
+					}
+					prev_case = 0;
 				}
-
-				prevCase = thisCase;
-				
-				++i;
-			}
-			
-			// Filter out empty parts
-			ArrayList partsArray = new ArrayList ();
-			foreach (string str in line.Split (' ')) {
-				if (str != "")
-					partsArray.Add (str);
 			}
 
-			// Assemble the array to return
-			string[] parts = new string [partsArray.Count];
-			for (i = 0; i < partsArray.Count; ++i)
-				parts [i] = (string) partsArray [i];
-			return parts;
+			return builder.ToString ();
 		}
 		
 		// Match strings against patterns that are allowed to contain
@@ -293,6 +304,20 @@ namespace Beagle.Util {
 			return builder.ToString ();
 		}
 
+		// Translate all %xx codes into real characters
+		static public string HexUnescape (string str)
+		{
+			int i = 0, pos = 0;
+			while ((i = str.IndexOf ('%', pos)) != -1) {
+				pos = i;
+				char unescaped = Uri.HexUnescape (str, ref pos);
+				str = str.Remove (i, 3);
+				str = str.Insert (i, new String(unescaped, 1));
+				pos -= 2;
+			}
+			return str;
+		}
+
 		static public string PathToQuotedFileUri (string path)
 		{
 			path = Path.GetFullPath (path);
@@ -473,5 +498,22 @@ namespace Beagle.Util {
 		{
 			return CountWords (str, -1);
 		}
+
+		// Strip trailing slashes and make sure we only have 1 leading slash
+		static public string SanitizePath (string path)
+		{
+			if (path.StartsWith ("//")) {
+				int pos;
+				for (pos = 2; pos < path.Length; pos++)
+					if (path [pos] != '/')
+						break;
+
+				path = path.Substring (pos - 1);
+			}
+			if (!(path.Length == 1 && path [0] == '/'))
+				path = path.TrimEnd ('/');
+
+			return path;
+		}
 	}
 }
