@@ -47,6 +47,7 @@ namespace Beagle.Daemon {
 
 		private static bool arg_replace = false;
 		private static bool arg_disable_scheduler = false;
+		private static bool arg_indexing_test_mode = false;
 
 		public static bool StartServer ()
 		{
@@ -71,7 +72,7 @@ namespace Beagle.Daemon {
 				request.Send ();
 				// Give it a second to shut down the messaging server
 				Thread.Sleep (1000);
-			} while (! StartServer ());
+			} while (! StartServer ());			
 		}
 
 		private static void LogMemoryUsage ()
@@ -107,6 +108,7 @@ namespace Beagle.Daemon {
 				"  --replace\t\tReplace a running daemon with a new instance.\n" +
 				"  --debug\t\tWrite out debugging information.\n" +
 				"  --debug-memory\tWrite out debugging information about memory use.\n" +
+				"  --indexing-test-mode\tRun in foreground, and exit when fully indexed.\n" + 
 				"  --deny-backend\tDeny a specific backend.\n" +
 				"  --allow-backend\tAllow a specific backend.\n" +
 				"  --list-backends\tList all the available backends.\n" +
@@ -117,10 +119,9 @@ namespace Beagle.Daemon {
 #if ENABLE_WEBSERVICES
 			usage += "\n" +
 				"  --web-global\t\tAllow global access to the Web & WebService interfaces.\n" +
-				"  --web-start\t\tStart internal web server for this instance.\n" +
 				"  --web-port\t\tPort to use for the internal web server.\n" +
 				"  --web-root\t\tRoot directory to use for the internal web server.\n" +
-				"  --enable-network\tStart internal web server & allow global access.\n";
+				"  --web-disable\t\tDisable Web & WebServices functionality.\n";			
 #endif 
 
 			Console.WriteLine (usage);
@@ -137,9 +138,14 @@ namespace Beagle.Daemon {
 			// Fire up our server
 			if (! StartServer ()) {
 				if (arg_replace)
+				{
+#if ENABLE_WEBSERVICES
+					WebServiceBackEnd.Stop();
+#endif			
 					ReplaceExisting ();
+				}		
 				else {
-					Logger.Log.Fatal ("Could not set up the listener for beagle requests.  "
+					Logger.Log.Error ("Could not set up the listener for beagle requests.  "
 							  + "There is probably another beagled instance running.  "
 							  + "Use --replace to replace the running service");
 					Environment.Exit (1);
@@ -150,6 +156,12 @@ namespace Beagle.Daemon {
 			if (Environment.GetEnvironmentVariable ("BEAGLE_ENABLE_IN_PROCESS_INDEXING") == null)
 				LuceneQueryable.IndexerHook = new LuceneQueryable.IndexerCreator (RemoteIndexer.NewRemoteIndexer);
 
+			// Initialize syncronization to keep the indexes local if PathFinder.HomeDir
+			// is on a non-block device, or if BEAGLE_SYNCHRONIZE_LOCALLY is set
+			if ((! SystemInformation.IsPathOnBlockDevice (PathFinder.HomeDir) && Conf.Daemon.IndexSynchronization) ||
+			    Environment.GetEnvironmentVariable ("BEAGLE_SYNCHRONIZE_LOCALLY") != null)
+				IndexSynchronization.Initialize ();
+
 			// Start the query driver.
 			Logger.Log.Debug ("Starting QueryDriver");
 			QueryDriver.Start ();
@@ -178,11 +190,35 @@ namespace Beagle.Daemon {
 			stopwatch.Stop ();
 
 			Logger.Log.Debug ("Daemon initialization finished after {0}", stopwatch);
-		
+
+			if (arg_indexing_test_mode) {
+				Thread.Sleep (1000); // Ugly paranoia: wait a second for the backends to settle.
+				Logger.Log.Debug ("Running in indexing test mode");
+				Scheduler.Global.EmptyQueueEvent += OnEmptySchedulerQueue;
+				Scheduler.Global.Add (null); // pulse the scheduler
+			}
 			return false;
 		}
 
-		public static int Main (string[] args)
+		static void OnEmptySchedulerQueue ()
+		{
+			Logger.Log.Debug ("Scheduler queue is empty: terminating immediately");
+			Shutdown.BeginShutdown ();
+			Environment.Exit (0); // Ugly work-around: We need to call Exit here to avoid deadlocking.
+		}
+
+		public static void Main (string[] args)
+		{
+			try {
+				DoMain (args);
+			} catch (Exception ex) {
+				Logger.Log.Error ("Unhandled exception thrown.  Exiting immediately.");
+				Logger.Log.Error (ex);
+				Environment.Exit (1);
+			}
+		}
+
+		public static void DoMain (string[] args)
 		{
 			// Process the command-line arguments
 			bool arg_debug = false;
@@ -203,6 +239,11 @@ namespace Beagle.Daemon {
 					Environment.Exit (0);
 					break;
 
+				case "--heap-buddy":
+					// Silently ignore the --heap-buddy argument: it gets handled
+					// in the wrapper script.
+					break;
+
 				case "--list-backends":
 					Console.WriteLine ("Current available backends:");
 					Console.Write (QueryDriver.ListBackends ());
@@ -231,6 +272,11 @@ namespace Beagle.Daemon {
 					arg_debug = true;
 					arg_debug_memory = true;
 					break;
+					
+				case "--indexing-test-mode":
+					arg_indexing_test_mode = true;
+					arg_fg = true;
+					break;
 
 				case "--allow-backend":
 					if (next_arg != null)
@@ -253,13 +299,17 @@ namespace Beagle.Daemon {
 				case "--disable-scheduler":
 					arg_disable_scheduler = true;
 					break;
+
+				case "--autostarted":
+					if (! Conf.Searching.Autostart) {
+						Console.WriteLine ("Autostarting is disabled, not starting");
+						Environment.Exit (0);
+					}
+					break;
 #if ENABLE_WEBSERVICES
 				case "--web-global":
 					WebServiceBackEnd.web_global = true;
-					break;
-
-				case "--web-start":
-					WebServiceBackEnd.web_start = true;
+					WebServiceBackEnd.web_start = true;					
 					break;
 
 				case "--web-port":
@@ -274,10 +324,9 @@ namespace Beagle.Daemon {
 					WebServiceBackEnd.web_start = true;
 					break;
 					
-				case "--enable-network":
-					WebServiceBackEnd.web_start = true;
-					WebServiceBackEnd.web_global = true;
-					break;					
+				case "--web-disable":
+					WebServiceBackEnd.web_start = false;
+					break;				
 #endif 
 				default:
 					Console.WriteLine ("Ignoring unknown argument '{0}'", arg);
@@ -286,6 +335,11 @@ namespace Beagle.Daemon {
 				}
 			}
 
+			if (arg_indexing_test_mode) {
+				LuceneQueryable.OptimizeRightAway = true;
+			}
+				
+
 			// Bail out if we are trying to run as root
 			if (Environment.UserName == "root") {
 				Console.WriteLine ("You can not run beagle as root.");
@@ -295,21 +349,18 @@ namespace Beagle.Daemon {
 
 			MainLoopThread = Thread.CurrentThread;
 
-			// Initialize logging.
-			// If we saw the --debug arg, set the default logging level
-			// accordingly.
-			if (arg_debug)
-				Logger.DefaultLevel = LogLevel.Debug;
-
-			Logger.LogToFile (PathFinder.LogDir, "Beagle", arg_fg);
+			Log.Initialize (PathFinder.LogDir,
+					"Beagle", 
+					// FIXME: We always turn on full debugging output!  We are still
+					// debugging this code, after all...
+					//arg_debug ? LogLevel.Debug : LogLevel.Warn,
+					LogLevel.Debug,
+					arg_fg);
 
 			Logger.Log.Info ("Starting Beagle Daemon (version {0})", ExternalStringsHack.Version);
 			
-			// FIXME: This try/catch is to work around a bug in mono 1.1.3
-			try {
-				Logger.Log.Debug ("Command Line: {0}",
-						  Environment.CommandLine != null ? Environment.CommandLine : "(null)");
-			} catch (Exception ex) { }
+			Logger.Log.Debug ("Command Line: {0}",
+					  Environment.CommandLine != null ? Environment.CommandLine : "(null)");
 
 			if (! ExtendedAttribute.Supported) {
 				Logger.Log.Warn ("Extended attributes are not supported on this filesystem.  " +
@@ -335,38 +386,35 @@ namespace Beagle.Daemon {
 
 			Application.Run ();
 
+			// If we placed our sockets in a temp directory, try to clean it up
+			// Note: this may fail because the helper is still running
+			if (PathFinder.GetRemoteStorageDir (false) != PathFinder.StorageDir) {
+				try {
+					Directory.Delete (PathFinder.GetRemoteStorageDir (false));
+				} catch (IOException) { }
+			}
+
 			Logger.Log.Debug ("Leaving BeagleDaemon.Main");
 
-			if (Logger.Log.Level == LogLevel.Debug) {
+			if (arg_debug) {
 				Thread.Sleep (500);
 				ExceptionHandlingThread.SpewLiveThreads ();
 			}
-
-			return 0;
 		}
 		
 
 		/////////////////////////////////////////////////////////////////////////////
 
-		// The integer values of the Mono.Posix.Signal enumeration don't actually
-		// match the Linux signal numbers of Linux.  Oops!
-		// This is fixed in Mono.Unix, but for the moment we want to maintain
-		// compatibility with mono 1.0.x.
-		const int ACTUAL_LINUX_SIGINT  = 2;
-		const int ACTUAL_LINUX_SIGQUIT = 3;
-		const int ACTUAL_LINUX_SIGTERM = 15;
-
 		static void SetupSignalHandlers ()
 		{
 			// Force OurSignalHandler to be JITed
 			OurSignalHandler (-1);
 
 			// Set up our signal handler
-			Mono.Posix.Syscall.sighandler_t sig_handler;
-			sig_handler = new Mono.Posix.Syscall.sighandler_t (OurSignalHandler);
-                        Mono.Posix.Syscall.signal (ACTUAL_LINUX_SIGINT, sig_handler);
-                        Mono.Posix.Syscall.signal (ACTUAL_LINUX_SIGQUIT, sig_handler);
-                        Mono.Posix.Syscall.signal (ACTUAL_LINUX_SIGTERM, sig_handler);
+			Mono.Unix.Stdlib.signal (Mono.Unix.Signum.SIGINT, OurSignalHandler);
+			Mono.Unix.Stdlib.signal (Mono.Unix.Signum.SIGTERM, OurSignalHandler);
+			if (Environment.GetEnvironmentVariable("BEAGLE_THERE_BE_NO_QUITTIN") == null)
+				Mono.Unix.Stdlib.signal (Mono.Unix.Signum.SIGQUIT, OurSignalHandler);
 		}
 
 		// Our handler triggers an orderly shutdown when it receives a signal.
