@@ -38,6 +38,7 @@ class ExtractContentTool {
 
 	static bool tokenize = false;
 	static bool show_children = false;
+	static string mime_type = null;
 
 	// FIXME: We don't display structural breaks
 	static void DisplayContent (string line)
@@ -69,16 +70,15 @@ class ExtractContentTool {
 
 		Console.WriteLine ("Filename: " + indexable.Uri);
 
-		if (indexable.MimeType != null)
-			Console.WriteLine ("MimeType: " + indexable.MimeType);
-
 		Filter filter;
 
 		if (! FilterFactory.FilterIndexable (indexable, out filter))
-			Console.WriteLine ("No filter!");
+			Console.WriteLine ("No filter for {0}", indexable.MimeType);
 
-		if (filter != null)
+		if (filter != null) {
 			Console.WriteLine ("Filter: {0}", filter);
+			Console.WriteLine ("MimeType: {0}", filter.MimeType);
+		}
 
 		Console.WriteLine ();
 
@@ -91,10 +91,14 @@ class ExtractContentTool {
 			Console.WriteLine ();
 		}
 
-		bool first;
+		// Make sure that the properties are sorted.
+		ArrayList prop_array = new ArrayList (indexable.Properties);
+		prop_array.Sort ();
 
+		bool first;
 		first = true;
-		foreach (Beagle.Property prop in indexable.Properties) {
+
+		foreach (Beagle.Property prop in prop_array) {
 			if (first) {
 				Console.WriteLine ("Properties:");
 				first = false;
@@ -163,16 +167,18 @@ class ExtractContentTool {
 		Console.WriteLine ("Usage: beagle-extract-content [OPTIONS] file [file ...]");
 		Console.WriteLine ();
 		Console.WriteLine ("Options:");
-		Console.WriteLine ("  --tokenize\t\tTokenize the text before printing");
-		Console.WriteLine ("  --show-children\tShow filtering information for items created by filters");
-		Console.WriteLine ("  --help\t\tShow this message");
+		Console.WriteLine ("  --debug\t\t\tPrint debug info to the console");
+		Console.WriteLine ("  --tokenize\t\t\tTokenize the text before printing");
+		Console.WriteLine ("  --show-children\t\tShow filtering information for items created by filters");
+		Console.WriteLine ("  --mimetype=<mime_type>\tUse filter for mime_type");
+		Console.WriteLine ("  --help\t\t\tShow this message");
 		Console.WriteLine ();
 	}
 
 	static int Main (string[] args)
 	{
-		Logger.DefaultEcho = true;
-		Logger.DefaultLevel = LogLevel.Debug;
+		if (Array.IndexOf (args, "--debug") == -1)
+			Log.Disable ();
 
 		if (Array.IndexOf (args, "--help") != -1) {
 			PrintUsage ();
@@ -187,12 +193,17 @@ class ExtractContentTool {
 
 		foreach (string arg in args) {
 
-			// option, skip it
-			if (arg.Substring (0, 2) == "--")
+			// mime-type option
+			if (arg.StartsWith ("--mimetype=")) {
+				mime_type = arg.Substring (11);    
 				continue;
-
+			} else if (arg.StartsWith ("--")) // option, skip it 
+				continue;
+			
 			Uri uri = UriFu.PathToFileUri (arg);
 			Indexable indexable = new Indexable (uri);
+			if (mime_type != null)
+				indexable.MimeType = mime_type;
 
 			try {
 				Display (indexable);
