@@ -44,13 +44,15 @@ namespace Beagle.Daemon.FileSystemQueryable {
 		private const string SplitFilenamePropKey = "beagle:Filename";
 		public const string ExactFilenamePropKey = "beagle:ExactFilename";
 		public const string TextFilenamePropKey = "beagle:TextFilename";
+		public const string FilenameExtensionPropKey = "beagle:FilenameExtension";
 		public const string ParentDirUriPropKey = LuceneQueryingDriver.PrivateNamespace + "ParentDirUri";
 		public const string IsDirectoryPropKey = LuceneQueryingDriver.PrivateNamespace + "IsDirectory";
 
 		// History:
 		// 1: Initially set to force a reindex due to NameIndex changes.
 		// 2: Overhauled everything to use new lucene infrastructure.
-		const int MINOR_VERSION = 2;
+		// 3: Switched to UTC for all times, changed the properties a bit.
+		const int MINOR_VERSION = 3;
 
 		private object big_lock = new object ();
 
@@ -136,19 +138,26 @@ namespace Beagle.Daemon.FileSystemQueryable {
 								     Guid      parent_id,
 								     bool      mutable)
 		{
+			string no_ext, ext;
+			no_ext = Path.GetFileNameWithoutExtension (name);
+			ext = Path.GetExtension (name);
+
 			Property prop;
 
 			prop = Property.NewKeyword (ExactFilenamePropKey, name);
 			prop.IsMutable = mutable;
 			indexable.AddProperty (prop);
 
-			prop = Property.New (TextFilenamePropKey, name);
+			prop = Property.New (TextFilenamePropKey, no_ext);
+			prop.IsMutable = mutable;
+			indexable.AddProperty (prop);
+
+			prop = Property.NewKeyword (FilenameExtensionPropKey, ext);
 			prop.IsMutable = mutable;
 			indexable.AddProperty (prop);
 
 			string str;
-			str = Path.GetFileNameWithoutExtension (name);
-			str = StringFu.FuzzyDivide (str);
+			str = StringFu.FuzzyDivide (no_ext);
 			prop = Property.New (SplitFilenamePropKey, str);
 			prop.IsMutable = mutable;
 			indexable.AddProperty (prop);
@@ -185,7 +194,7 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			indexable = new Indexable (IndexableType.Add, GuidFu.ToUri (id));
 			indexable.MimeType = "inode/directory";
 			indexable.NoContent = true;
-			indexable.Timestamp = Directory.GetLastWriteTime (path);
+			indexable.Timestamp = Directory.GetLastWriteTimeUtc (path);
 
 			string name;
 			if (parent == null)
@@ -211,7 +220,7 @@ namespace Beagle.Daemon.FileSystemQueryable {
 		{
 			Indexable indexable;
 			indexable = new Indexable (IndexableType.Add, GuidFu.ToUri (id));
-			indexable.Timestamp = File.GetLastWriteTime (path);
+			indexable.Timestamp = File.GetLastWriteTimeUtc (path);
 			indexable.ContentUri = UriFu.PathToFileUri (path);
 			indexable.Crawled = crawl_mode;
 			indexable.Filtering = Beagle.IndexableFiltering.Always;
@@ -537,7 +546,7 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			else
 				dir = parent.AddChild (name, attr);
 
-			if (Directory.GetLastWriteTime (path) > attr.LastWriteTime) {
+			if (Directory.GetLastWriteTimeUtc (path) > attr.LastWriteTime) {
 				dir.State = DirectoryState.Dirty;
 				if (Debug)
 					Logger.Log.Debug ("'{0}' is dirty", path);
@@ -702,11 +711,11 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			// we only use the FileAttributes mtime on a directory
 			// to determine its initial state, not whether or not
 			// its index record is up-to-date.
-			attr.LastWriteTime = DateTime.Now;
+			attr.LastWriteTime = DateTime.UtcNow;
 
 			// ...but we do use this to decide which order directories get
 			// crawled in.
-			dir.LastCrawlTime = DateTime.Now;
+			dir.LastCrawlTime = DateTime.UtcNow;
 
 			FileAttributesStore.Write (attr);
 			dir.MarkAsClean ();
@@ -801,15 +810,20 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			Rename,
 			Forget
 		}
+
+		static DateTime epoch = new DateTime (1970, 1, 1, 0, 0, 0);
+
+		static DateTime ToDateTimeUtc (long time_t)
+		{
+			return epoch.AddSeconds (time_t);
+		}
 		
 		private RequiredAction DetermineRequiredAction (DirectoryModel dir,
 								string         name,
 								FileAttributes attr,
-								out string     last_known_path,
-								out DateTime   mtime)
+								out string     last_known_path)
 		{
 			last_known_path = null;
-			mtime = DateTime.MinValue;
 
 			string path;
 			path = Path.Combine (dir.FullName, name);
@@ -850,19 +864,22 @@ namespace Beagle.Daemon.FileSystemQueryable {
 				}
 			}
 
-			Mono.Posix.Stat stat;
+			Mono.Unix.Stat stat;
 			try {
-				Mono.Posix.Syscall.stat (path, out stat);
+				Mono.Unix.Syscall.stat (path, out stat);
 			} catch (Exception ex) {
 				Logger.Log.Debug ("Caught exception stat-ing {0}", path);
 				Logger.Log.Debug (ex);
 				return RequiredAction.None;
 			}
-			mtime = stat.MTime;
 
-			if (! DatesAreTheSame (attr.LastWriteTime, mtime)) {
+			DateTime last_write_time, last_attr_time;
+			last_write_time = ToDateTimeUtc (stat.st_mtime);
+			last_attr_time = ToDateTimeUtc (stat.st_ctime);
+
+			if (attr.LastWriteTime != last_write_time) {
 				if (Debug)
-					Logger.Log.Debug ("*** Index it: MTime has changed");
+					Logger.Log.Debug ("*** Index it: MTime has changed ({0} vs {1})", attr.LastWriteTime, last_write_time);
 				
 				// If the file has been copied, it will have the
 				// original file's EAs.  Thus we have to check to
@@ -883,9 +900,9 @@ namespace Beagle.Daemon.FileSystemQueryable {
 
 			// If the inode ctime is different that the time we last
 			// set file attributes, we might have been moved or copied.
-			if (! DatesAreTheSame (attr.LastAttrTime, stat.CTime)) {
+			if (attr.LastAttrTime != last_attr_time) {
 				if (Debug)
-					Logger.Log.Debug ("*** CTime has changed, checking last known path");
+					Logger.Log.Debug ("*** CTime has changed, checking last known path ({0} vs {1})", attr.LastAttrTime, last_attr_time);
 
 				last_known_path = UniqueIdToFullPath (attr.UniqueId);
 
@@ -911,18 +928,6 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			return RequiredAction.None;	
 		}
 
-		// This works around a mono bug: the DateTimes that we get out of stat
-		// don't correctly account for daylight savings time.  We declare the two
-		// dates to be equal if:
-		// (1) They actually are equal
-		// (2) The first date is exactly one hour ahead of the second
-		static private bool DatesAreTheSame (DateTime system_io_datetime, DateTime stat_datetime)
-		{
-			const double epsilon = 1e-5;
-			double t = (system_io_datetime - stat_datetime).TotalSeconds;
-			return Math.Abs (t) < epsilon || Math.Abs (t-3600) < epsilon;
-		}
-
 		// Return an indexable that will do the right thing with a file
 		// (or null, if the right thing is to do nothing)
 		public Indexable GetCrawlingFileIndexable (DirectoryModel dir, string name)
@@ -935,8 +940,7 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			
 			RequiredAction action;
 			string last_known_path;
-			DateTime mtime;
-			action = DetermineRequiredAction (dir, name, attr, out last_known_path, out mtime);
+			action = DetermineRequiredAction (dir, name, attr, out last_known_path);
 
 			if (action == RequiredAction.None)
 				return null;
@@ -953,8 +957,6 @@ namespace Beagle.Daemon.FileSystemQueryable {
 
 			case RequiredAction.Index:
 				indexable = FileToIndexable (path, unique_id, dir, true);
-				if (mtime == DateTime.MinValue)
-					mtime = File.GetLastWriteTime (path);
 				break;
 
 			case RequiredAction.Rename:
