@@ -175,8 +175,10 @@ namespace Beagle.Daemon {
 		{
 			int count = 0;
 
-			foreach (string path in Conf.Daemon.StaticQueryables)
-				static_queryables.Add (path);
+			if (UseQueryable ("static")) {
+				foreach (string path in Conf.Daemon.StaticQueryables)
+					static_queryables.Add (path);
+			}
 
 			foreach (string path in static_queryables) {
 				DirectoryInfo index_dir = new DirectoryInfo (path);
@@ -284,11 +286,8 @@ namespace Beagle.Daemon {
 				Server.ScanAssemblyForExecutors (assembly);
 			}
 
-			// FIXME: This is not nice at all
-			if (UseQueryable ("system"))
-				LoadSystemIndexes ();
-			if (UseQueryable ("static"))
-				LoadStaticQueryables ();
+			LoadSystemIndexes ();
+			LoadStaticQueryables ();
 
 			foreach (Queryable q in queryables)
 				q.Start ();
@@ -342,11 +341,36 @@ namespace Beagle.Daemon {
 
 		////////////////////////////////////////////////////////
 
+		private class MarkAndForwardHits : IQueryResult {
+
+			IQueryResult result;
+			string name;
+			
+			public MarkAndForwardHits (IQueryResult result, string name)
+			{
+				this.result = result;
+				this.name = name;
+			}
+
+			public void Add (ICollection some_hits)
+			{
+				foreach (Hit hit in some_hits)
+					if (hit != null)
+						hit.SourceObjectName = name;
+				result.Add (some_hits);
+			}
+
+			public void Subtract (ICollection some_uris)
+			{
+				result.Subtract (some_uris);
+			}
+		}
+
 		private class QueryClosure : IQueryWorker {
 
 			Queryable queryable;
 			Query query;
-			QueryResult result;
+			IQueryResult result;
 			IQueryableChangeData change_data;
 			
 			public QueryClosure (Queryable            queryable,
@@ -356,16 +380,13 @@ namespace Beagle.Daemon {
 			{
 				this.queryable = queryable;
 				this.query = query;
-				this.result = result;
+				this.result = new MarkAndForwardHits (result, queryable.Name);
 				this.change_data = change_data;
 			}
 
 			public void DoWork ()
 			{
-				HitRegulator regulator = result.GetHitRegulator (queryable);
-				regulator.MaxHits = query.MaxHits;
-				queryable.DoQuery (query, regulator, change_data);
-				regulator.Flush (result);
+				queryable.DoQuery (query, result, change_data);
 			}
 		}
 
@@ -380,7 +401,104 @@ namespace Beagle.Daemon {
 			}
 		}
 
-		static public void DoQuery (Query query, QueryResult result)
+		static void AddSearchTermInfo (QueryPart          part,
+					       SearchTermResponse response, StringBuilder sb)
+		{
+			if (part.Logic == QueryPartLogic.Prohibited)
+				return;
+
+			if (part is QueryPart_Or) {
+				ICollection sub_parts;
+				sub_parts = ((QueryPart_Or) part).SubParts;
+				foreach (QueryPart qp in sub_parts)
+					AddSearchTermInfo (qp, response, sb);
+				return;
+			}
+
+			if (! (part is QueryPart_Text))
+				return;
+
+			QueryPart_Text tp;
+			tp = (QueryPart_Text) part;
+
+			string [] split;
+			split = tp.Text.Split (' ');
+ 
+			// First, remove stop words
+			for (int i = 0; i < split.Length; ++i)
+				if (LuceneCommon.IsStopWord (split [i]))
+					split [i] = null;
+
+			// Assemble the phrase minus stop words
+			sb.Length = 0;
+			for (int i = 0; i < split.Length; ++i) {
+				if (split [i] == null)
+					continue;
+				if (sb.Length > 0)
+					sb.Append (' ');
+				sb.Append (split [i]);
+			}
+			response.ExactText.Add (sb.ToString ());
+
+			// Now assemble a stemmed version
+			sb.Length = 0; // clear the previous value
+			for (int i = 0; i < split.Length; ++i) {
+				if (split [i] == null)
+					continue;
+				if (sb.Length > 0)
+					sb.Append (' ');
+				sb.Append (LuceneCommon.Stem (split [i]));
+			}
+			response.StemmedText.Add (sb.ToString ());
+		}
+
+		////////////////////////////////////////////////////////
+
+		static private void DehumanizeQuery (Query query)
+		{
+			// We need to remap any QueryPart_Human parts into
+			// lower-level part types.  First, we find any
+			// QueryPart_Human parts and explode them into
+			// lower-level types.
+			ArrayList new_parts = null;
+			foreach (QueryPart abstract_part in query.Parts) {
+				if (abstract_part is QueryPart_Human) {
+					QueryPart_Human human = abstract_part as QueryPart_Human;
+					if (new_parts == null)
+						new_parts = new ArrayList ();
+					foreach (QueryPart sub_part in QueryStringParser.Parse (human.QueryString))
+						new_parts.Add (sub_part);
+				}
+			}
+
+			// If we found any QueryPart_Human parts, copy the
+			// non-Human parts over and then replace the parts in
+			// the query.
+			if (new_parts != null) {
+				foreach (QueryPart abstract_part in query.Parts) {
+					if (! (abstract_part is QueryPart_Human))
+						new_parts.Add (abstract_part);
+				}
+				
+				query.ClearParts ();
+				foreach (QueryPart part in new_parts)
+					query.AddPart (part);
+			}
+
+		}
+
+		static private SearchTermResponse AssembleSearchTermResponse (Query query)
+		{
+			StringBuilder sb = new StringBuilder ();
+			SearchTermResponse search_term_response;
+			search_term_response = new SearchTermResponse ();
+			foreach (QueryPart part in query.Parts)
+				AddSearchTermInfo (part, search_term_response, sb);
+			return search_term_response;
+		}
+
+		static private void QueryEachQueryable (Query       query,
+							QueryResult result)
 		{
 			// The extra pair of calls to WorkerStart/WorkerFinished ensures:
 			// (1) that the QueryResult will fire the StartedEvent
@@ -399,20 +517,63 @@ namespace Beagle.Daemon {
 			
 			result.WorkerFinished (dummy_worker);
 		}
+		
+		static public void DoQueryLocal (Query       query,
+						 QueryResult result)
+		{
+			DehumanizeQuery (query);
+
+			SearchTermResponse search_term_response;
+			search_term_response = AssembleSearchTermResponse (query);
+			query.ProcessSearchTermResponse (search_term_response);
+
+			QueryEachQueryable (query, result);
+		}
+
+		static public void DoQuery (Query                                query,
+					    QueryResult                          result,
+					    RequestMessageExecutor.AsyncResponse send_response)
+		{
+			DehumanizeQuery (query);
+
+			SearchTermResponse search_term_response;
+			search_term_response = AssembleSearchTermResponse (query);
+			send_response (search_term_response);
+
+			QueryEachQueryable (query, result);
+		}
 
 		////////////////////////////////////////////////////////
 
 		static public string GetIndexInformation ()
 		{
-			StringBuilder builder = new StringBuilder ("\n");
+			StringBuilder builder = new StringBuilder ('\n');
 
 			foreach (Queryable q in queryables) {
-				builder.AppendFormat ("Name: {0}\n", q.Name);
-				builder.AppendFormat ("Count: {0}\n", q.GetItemCount ());
-				builder.Append ("\n");
+				QueryableStatus status = q.GetQueryableStatus ();
+
+				builder.Append ("Name: ").Append (status.Name).Append ('\n');
+				builder.Append ("Count: ").Append (status.ItemCount).Append ('\n');
+				builder.Append ("Indexing: ").Append (status.IsIndexing).Append ('\n');
+				builder.Append ('\n');
 			}
 
 			return builder.ToString ();
 		}
+
+		////////////////////////////////////////////////////////
+
+		static public bool IsIndexing {
+			get {
+				foreach (Queryable q in queryables) {
+					QueryableStatus status = q.GetQueryableStatus ();
+
+					if (status.IsIndexing)
+						return true;
+				}
+
+				return false;
+			}
+		}					
 	}
 }
